from typing import Union
import functools
import time

import MetaTrader5 as _mt5

from .Errors import MT5Error
class MT5Wrappers:
    def load_mt5(fn):
        @functools.wraps(fn)
        def wrapper(*args, **kwargs):
            _mt5.initialize()
            _mt5.login()
            result = fn(*args, **kwargs)
            _mt5.shutdown()
            return result
        return wrapper

    def load_symbol(fn):
        @functools.wraps(fn)
        def wrapper(data: Union[str, dict], *args, **kwargs):
            symbol = data["symbol"] if isinstance(data, dict) else data
            loaded = _mt5.symbol_select(symbol)
            if not loaded:
                raise MT5Error(f"Couldn't load {symbol}")
            result = fn(data, *args, **kwargs)
            # Tries to deselect it, if it can't it's fine
            _mt5.symbol_select(symbol, False)
            return result
        return wrapper


import setuptools

VERSION = "0.0.4"
DESCRIPTION = "A user friendly version of the Metatrader5 python library."



with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="BetterMt5",
    version=VERSION,
    author="Mathijs Follon",
    author_email="contact@mfollon.com",
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mathlon26/BetterMt5",
    packages=[
        "BetterMt5",
    ],
    install_requires=[
        "MetaTrader5>=5.0.45"
    ],
    python_requires=">=3.6",
    include_package_data=True,
    keywords=['python', 'trading', 'trader', 'metatrader', 'MetaTrader5', 'daytrading', 'bot', 'expert advisor', 'EA', 'automated'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "License :: OSI Approved :: MIT License"
    ]
)



from typing import Union, Optional
from pathlib import Path

from datetime import datetime
from datetime import timedelta


import shutil
import subprocess
import time
import logging
import sys

from Errors import MT5Error
from Global import TIMEFRAME

import MetaTrader5 as _mt5
from .Decorators import MT5Wrappers as w

class MT5Utils:
    
    def normalize_path(self,
        path: Union[None, str, Path], 
        parent: Union[str, Path], 
        target: str
    ) -> str:
        """Takes a possible full path, partial path or None and
        turns it into the desired format."""
        parent = Path(parent)
        if path is None:
            return f"{str(parent.absolute())}{target}"
        else:
            path = Path(path)
            if path.is_dir():
                return f"{str(path.absolute())}{target}"
            else:
                return str(path.absolute())
    
    @w.load_mt5   
    def get_timzone_offset(self) -> float:
        """Function that calculated current timezone offset with broker server based on the difference
        between the last candle time and the current utc time in hours."""
        
        weekday = datetime.weekday(datetime.now())
        # if it is the weekend set the offset to 3 days
        if weekday in [4, 5, 6]:
            offset = 3 * 24 * 60 
        else:
            offset = 0 
            
        possible_symbols = _mt5.symbols_get()
        symbol = possible_symbols[0].name

        last_candle = _mt5.copy_rates_from_pos(symbol, TIMEFRAME.M1, offset, 1)
        last_candle_time = datetime.fromtimestamp(last_candle[0][0])
        reference = datetime.utcnow().replace(second=0, microsecond=0)
        reference -= timedelta(minutes=offset)
        offset = (last_candle_time - reference).total_seconds() / 3600

        return offset




import MetaTrader5 as mt5

class TIMEFRAME_SEC:
    """
    Timeframes in seconds given when requesting the simplified key of the MetaTrade5 lib. Eg. TIMEFRAME_SEC.M15 --> 900 (15 minutes in seconds)
    """
    M1 = 60
    M2 = 120
    M3 = 180
    M4 = 240
    M5 = 300
    M6 = 360
    M10 = 600
    M12 = 720
    M15 = 900
    M20 = 1200
    M30 = 1800
    H1 = 3600
    H2 = 7200
    H3 = 10800
    H4 = 14400
    H6 = 21600
    H8 = 28800
    H12 = 43200
    D1 = 86400
    W1 = 604800
    MN1 = 2592000


class TIMEFRAME:
    """
    Timeframes of the MetaTrade5 lib. Eg. TIMEFRAME.M15 --> MetaTrader5.TIMEFRAME_M15
    """
    M1 = mt5.TIMEFRAME_M1
    M2 = mt5.TIMEFRAME_M2
    M3 = mt5.TIMEFRAME_M3
    M4 = mt5.TIMEFRAME_M4
    M5 = mt5.TIMEFRAME_M5
    M6 = mt5.TIMEFRAME_M6
    M10 = mt5.TIMEFRAME_M10
    M12 = mt5.TIMEFRAME_M12
    M15 = mt5.TIMEFRAME_M15
    M20 = mt5.TIMEFRAME_M20
    M30 = mt5.TIMEFRAME_M30
    H1 = mt5.TIMEFRAME_H1
    H2 = mt5.TIMEFRAME_H2
    H3 = mt5.TIMEFRAME_H3
    H4 = mt5.TIMEFRAME_H4
    H6 = mt5.TIMEFRAME_H6
    H8 = mt5.TIMEFRAME_H8
    H12 = mt5.TIMEFRAME_H12
    D1 = mt5.TIMEFRAME_D1
    W1 = mt5.TIMEFRAME_W1
    MN1 = mt5.TIMEFRAME_MN1

class ORDER_TYPES:
    BUY = mt5.ORDER_TYPE_BUY
    SELL = mt5.ORDER_TYPE_SELL
    BUY_LIMIT = mt5.ORDER_TYPE_BUY_LIMIT
    SELL_LIMIT = mt5.ORDER_TYPE_SELL_LIMIT
    BUY_STOP = mt5.ORDER_TYPE_BUY_STOP
    SELL_STOP = mt5.ORDER_TYPE_SELL_STOP
    BUY_STOP_LIMIT = mt5.ORDER_TYPE_BUY_STOP_LIMIT
    SELL_STOP_LIMIT = mt5.ORDER_TYPE_SELL_STOP_LIMIT
    CLOSE = mt5.ORDER_TYPE_CLOSE_BY
